#!/usr/bin/snimpy

"""
Get serial number of a given equipment using ENTITY-MIB
"""

import sys

load("ENTITY-MIB")

host=sys.argv[1]
s = M(host=host, community=sys.argv[2])

# Locate parent of all other elements
print("[-] %s: Search for parent element" % host)
parent = None
for i in s.entPhysicalContainedIn:
    if s.entPhysicalContainedIn[i] == 0:
        parent = i
        break
if parent is None:
    print("[!] %s: Unable to find parent" % host)
    sys.exit(1)
print("[+] {}: {}".format(host, s.entPhysicalDescr[parent]))
print("[+] {}: HW {}, FW {}, SW {}".format(host,
                                           s.entPhysicalHardwareRev[parent],
                                           s.entPhysicalFirmwareRev[parent],
                                           s.entPhysicalSoftwareRev[parent]))
print("[+] {}: SN {}".format(host,
                             s.entPhysicalSerialNum[parent]))



import os
import time
from datetime import timedelta
from snimpy.manager import load, Manager, snmp
import agent
import unittest


class TestManager(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        load('IF-MIB')
        load('SNMPv2-MIB')
        load(os.path.join(os.path.dirname(os.path.abspath(__file__)),
                          "SNIMPY-MIB.mib"))
        cls.agent = agent.TestAgent()

    @classmethod
    def tearDownClass(cls):
        cls.agent.terminate()

    def setUp(self):
        self.manager = Manager(host="127.0.0.1:{}".format(self.agent.port),
                               community="public",
                               version=2)
        self.session = self.manager._session


class TestManagerGet(TestManager):

    """Test getting stuff from manager"""

    def testGetScalar(self):
        """Retrieve some simple scalar values"""
        self.assertEqual(self.manager.sysDescr, "Snimpy Test Agent public")
        self.assertEqual(self.manager.ifNumber, 3)

    def scalarGetAndCheck(self, name, value):
        self.assertEqual(getattr(self.manager, name),
                         value)

    def testScalar_IpAddress(self):
        """Retrieve IpAdress as a scalar"""
        self.scalarGetAndCheck("snimpyIpAddress", "65.65.65.65")

    def testScalar_String(self):
        """Retrieve a String as a scalar"""
        self.scalarGetAndCheck("snimpyString", "bye")

    def testScalar_Integer(self):
        """Retrieve an Integer as a scalar"""
        self.scalarGetAndCheck("snimpyInteger", 19)

    def testScalar_Enum(self):
        """Retrieve an Enum as a scalar"""
        self.scalarGetAndCheck("snimpyEnum", "down")

    def testScalar_ObjectId(self):
        """Retrieve an ObjectId as a scalar"""
        self.scalarGetAndCheck("snimpyObjectId", (1, 3, 6, 4454, 0, 0))

    def testScalar_Boolean(self):
        """Retrieve a Boolean as a scalar"""
        self.scalarGetAndCheck("snimpyBoolean", True)

    def testScalar_Counter(self):
        """Retrieve a Counter as a scalar"""
        self.scalarGetAndCheck("snimpyCounter", 47)
        self.scalarGetAndCheck("snimpyCounter64", 2 ** 48 + 3)

    def testScalar_Gauge(self):
        """Retrieve a Gauge as a scalar"""
        self.scalarGetAndCheck("snimpyGauge", 18)

    def testScalar_Timeticks(self):
        """Retrieve a TimeTicks as a scalar"""
        self.scalarGetAndCheck(
            "snimpyTimeticks",
            timedelta(days=1,
                      hours=9,
                      minutes=38,
                      seconds=31))

    def testScalar_Bits(self):
        """Retrieve Bits as a scalar"""
        self.scalarGetAndCheck("snimpyBits", ["first", "third", "secondByte"])

    def testScalar_MacAddress(self):
        """Retrieve MacAddress as a scalar"""
        self.scalarGetAndCheck("snimpyMacAddress", "11:12:13:14:15:16")

    def testContains_IfDescr(self):
        """Test proxy column membership checking code"""
        self.assertEqual(2 in self.manager.ifDescr,
                         True)
        # FIXME: this currently fails under TestManagerWithNone
        # self.assertEqual(10 in self.manager.ifDescr,
        #                 False)

    def testWalkIfDescr(self):
        """Test we can walk IF-MIB::ifDescr and IF-MIB::ifTpe"""
        results = [(idx, self.manager.ifDescr[idx], self.manager.ifType[idx])
                   for idx in self.manager.ifIndex]
        self.assertEqual(results,
                         [(1, "lo", 24),
                          (2, "eth0", 6),
                          (3, "eth1", 6)])

    def testWalkIfTable(self):
        """Test we can walk IF-MIB::ifTable"""
        results = [(idx, self.manager.ifDescr[idx], self.manager.ifType[idx])
                   for idx in self.manager.ifTable]
        self.assertEqual(results,
                         [(1, "lo", 24),
                          (2, "eth0", 6),
                          (3, "eth1", 6)])

    def testWalkNotAccessible(self):
        """Test we can walk a table with the first entry not accessible."""
        list(self.manager.ifRcvAddressTable)

    def testWalkIfDescrWithoutBulk(self):
        """Walk IF-MIB::ifDescr without GETBULK"""
        self.session.bulk = False
        self.testWalkIfDescr()

    def testWalkIfTableWithoutBulk(self):
        """Walk IF-MIB::ifTable without GETBULK"""
        self.session.bulk = False
        self.testWalkIfTable()

    def testWalkComplexIndexes(self):
        """Test if we can walk a table with complex indexes"""
        results = [(idx, self.manager.snimpyIndexInt[idx])
                   for idx in self.manager.snimpyIndexInt]
        self.assertEqual(results,
                         [(("row1", (1, 2, 3),
                            "alpha5", "end of row1"), 4571),
                          (("row2", (1, 0, 2, 3),
                            "beta32", "end of row2"), 78741),
                          (("row3", (120, 1, 2, 3),
                            "gamma7", "end of row3"), 4110)])

    def testWalkTableWithComplexIndexes(self):
        """Test if we can walk a table with complex indexes"""
        results = [(idx, self.manager.snimpyIndexInt[idx])
                   for idx in self.manager.snimpyIndexTable]
        self.assertEqual(results,
                         [(("row1", (1, 2, 3),
                            "alpha5", "end of row1"), 4571),
                          (("row2", (1, 0, 2, 3),
                            "beta32", "end of row2"), 78741),
                          (("row3", (120, 1, 2, 3),
                            "gamma7", "end of row3"), 4110)])

    def testWalkReuseIndexes(self):
        """Test if we can walk a table with re-used indexes"""
        results = [(idx, self.manager.snimpyReuseIndexValue[idx])
                   for idx in self.manager.snimpyReuseIndexValue]
        self.assertEqual(results,
                         [(("end of row1", 4), 1785),
                          (("end of row1", 5), 2458)])

    def testWalkTableWithReuseIndexes(self):
        """Test if we can walk a table with re-used indexes"""
        results = [(idx, self.manager.snimpyReuseIndexValue[idx])
                   for idx in self.manager.snimpyReuseIndexTable]
        self.assertEqual(results,
                         [(("end of row1", 4), 1785),
                          (("end of row1", 5), 2458)])

    def testWalkPartialIndexes(self):
        """Test if we can walk a slice of a table given a partial index"""
        results = [(idx, self.manager.ifRcvAddressType[idx])
                   for idx in self.manager.ifRcvAddressStatus[2]]
        self.assertEqual(results,
                         [((2, "61:62:63:64:65:66"), 1),
                          ((2, "67:68:69:6a:6b:6c"), 1)])
        results = [(idx, self.manager.ifRcvAddressType[idx])
                   for idx in self.manager.ifRcvAddressStatus[(3,)]]
        self.assertEqual(results,
                         [((3, "6d:6e:6f:70:71:72"), 1)])
        results = list(self.manager.ifRcvAddressType.iteritems(3))
        self.assertEqual(results,
                         [((3, "6d:6e:6f:70:71:72"), 1)])
        results = list(self.manager.ifRcvAddressType.items(3))
        self.assertEqual(results,
                         [((3, "6d:6e:6f:70:71:72"), 1)])

    def testWalkInvalidPartialIndexes(self):
        """Try to get a table slice with an incorrect index filter"""
        self.assertRaises(ValueError,
                          lambda: list(
                              self.manager.ifRcvAddressStatus.iteritems(
                                  (3, "6d:6e:6f:70:71:72"))))

    def testContains_Partial(self):
        """Test proxy column membership checking code with partial indexes"""
        self.assertEqual(
                "61:62:63:64:65:66" in self.manager.ifRcvAddressStatus[2],
                True)
        # FIXME: this currently fails under TestManagerWithNone
        # self.assertEqual(
        #        "6d:6e:6f:70:71:72" in self.manager.ifRcvAddressStatus[2],
        #        False)

    def testScalar_MultipleSubscripts(self):
        """Retrieve a scalar value using multiple subscript syntax
        (attr[x][y])"""
        self.assertEqual(self.manager.ifRcvAddressType[2]["67:68:69:6a:6b:6c"],
                         1)

    def testGetInexistentStuff(self):
        """Try to access stuff that does not exist on the agent"""
        self.assertRaises(snmp.SNMPNoSuchObject,
                          getattr, self.manager, "snimpyNotImplemented")
        self.assertRaises(snmp.SNMPNoSuchObject,
                          self.manager.ifName.__getitem__, 47)
        self.assertRaises(snmp.SNMPNoSuchInstance,
                          self.manager.ifDescr.__getitem__, 47)

    def testAccessInexistentStuff(self):
        """Try to access stuff that don't exist in MIB"""
        self.assertRaises(AttributeError,
                          getattr, self.manager, "iDoNotExist")

    def testAccessIncorrectIndex(self):
        """Try to access with incorrect indexes"""
        self.assertRaises(ValueError,
                          self.manager.ifDescr.__getitem__, (47, 18))
        self.assertRaises(ValueError,
                          self.manager.ifDescr.__getitem__, "nothing")

    def testAccessEmptyTable(self):
        """Try to walk an empty table"""
        results = [(idx,) for idx in self.manager.snimpyEmptyDescr]
        self.assertEqual(results, [])

    def testAccessNotExistentTable(self):
        """Try to walk a non-existent table"""
        agent2 = agent.TestAgent(emptyTable=False)
        try:
            manager = Manager(host="127.0.0.1:{}".format(agent2.port),
                              community="public",
                              version=2)
            [(idx,) for idx in manager.snimpyEmptyDescr]
        except snmp.SNMPNoSuchObject:
            pass                # That's OK
        else:
            self.assertFalse("should raise SNMPNoSuchObject exception")
        finally:
            agent2.terminate()

    def testGetChangingStuff(self):
        """Get stuff with varying values"""
        initial = self.manager.ifInOctets[2]
        current = self.manager.ifInOctets[2]
        self.assertGreater(current, initial)


class TestManagerRestrictModule(TestManager):

    """Test when we restrict modules to be used by a manager"""

    def testGetSpecificModule(self):
        """Get a scalar from a specific module only"""
        self.assertEqual(self.manager['IF-MIB'].ifNumber, 3)
        self.assertEqual(self.manager['SNMPv2-MIB'].sysDescr,
                         "Snimpy Test Agent public")

    def testGetInexistentModule(self):
        """Get a scalar from a non loaded module"""
        self.assertRaises(KeyError, lambda: self.manager['IF-MIB2'])

    def testGetInexistentScalarFromModule(self):
        """Get a non-existent scalar from a specific module"""
        self.assertRaises(AttributeError,
                          lambda: self.manager['IF-MIB'].sysDescr)


class TestManagerSet(TestManager):

    """Test setting stuff from manager"""

    def testSetScalar(self):
        """Try to set a simple value"""
        self.manager.snimpyString = "hello"
        self.assertEqual(self.manager.snimpyString, "hello")

    def scalarSetAndCheck(self, name, value):
        setattr(self.manager, name, value)
        self.assertEqual(getattr(self.manager, name),
                         value)

    def testScalar_IpAddress(self):
        """Retrieve IpAdress as a scalar"""
        self.scalarSetAndCheck("snimpyIpAddress", "165.255.65.65")

    def testScalar_String(self):
        """Retrieve a String as a scalar"""
        self.scalarSetAndCheck("snimpyString", "awesome !!!")

    def testScalar_Integer(self):
        """Retrieve an Integer as a scalar"""
        self.scalarSetAndCheck("snimpyInteger", 1900)

    def testScalar_Enum(self):
        """Retrieve an Enum as a scalar"""
        self.scalarSetAndCheck("snimpyEnum", "up")

    def testScalar_ObjectId(self):
        """Retrieve an ObjectId as a scalar"""
        self.scalarSetAndCheck("snimpyObjectId", (1, 3, 6, 4454, 19, 47))

    def testScalar_Boolean(self):
        """Retrieve a Boolean as a scalar"""
        self.scalarSetAndCheck("snimpyBoolean", False)

    def testScalar_Counter(self):
        """Retrieve a Counter as a scalar"""
        self.scalarSetAndCheck("snimpyCounter", 4700)
        self.scalarSetAndCheck("snimpyCounter64", 2 ** 48 + 3 - 18)

    def testScalar_Gauge(self):
        """Retrieve a Gauge as a scalar"""
        self.scalarSetAndCheck("snimpyGauge", 180014)

    def testScalar_Timeticks(self):
        """Retrieve a TimeTicks as a scalar"""
        self.scalarSetAndCheck(
            "snimpyTimeticks",
            timedelta(days=1,
                      hours=17,
                      minutes=38,
                      seconds=31))

    def testScalar_Bits(self):
        """Retrieve Bits as a scalar"""
        self.scalarSetAndCheck("snimpyBits", ["first", "second", "secondByte"])

    def testScalar_MacAddress(self):
        """Retrieve MAC address as a scala"""
        self.scalarSetAndCheck("snimpyMacAddress", "a0:b0:c0:d0:e:ff")

    def testNonScalarSet(self):
        """Check we can set a non-scalar value"""
        idx = ("row2", (1, 0, 2, 3), "beta32", "end of row2")
        self.manager.snimpyIndexInt[idx] = 1041
        self.assertEqual(self.manager.snimpyIndexInt[idx], 1041)

    def testSetWithContext(self):
        """Set several values atomically (inside a context)"""
        with self.manager as m:
            m.snimpyString = "Noooooo!"
            m.snimpyInteger = 42
        self.assertEqual(m.snimpyString, "Noooooo!")
        self.assertEqual(m.snimpyInteger, 42)

    def testSetWithContextAndAbort(self):
        """Check if writing several values atomically can be aborted"""
        try:
            with self.manager as m:
                m.snimpyString = "Abort sir!"
                m.snimpyInteger = 37
                raise RuntimeError("Abort now!")
        except RuntimeError as e:
            self.assertEqual(str(e), "Abort now!")
        self.assertNotEqual(m.snimpyString, "Abort sir!")
        self.assertNotEqual(m.snimpyInteger, 37)

    def testSetInexistentStuff(self):
        """Try to access stuff that does not exist on the agent"""
        self.assertRaises(snmp.SNMPNotWritable,
                          setattr, self.manager, "snimpyNotImplemented",
                          "Hello")
        self.assertRaises(snmp.SNMPNotWritable,
                          self.manager.ifName.__setitem__, 47, "Wouh")
        self.assertRaises(snmp.SNMPNotWritable,
                          self.manager.ifDescr.__setitem__, 47, "Noooo")

    def testAccessInexistentStuff(self):
        """Try to access stuff that don't exist in MIB"""
        self.assertRaises(AttributeError,
                          setattr, self.manager, "iDoNotExist", 47)

    def testAccessIncorrectIndex(self):
        """Try to access with incorrect indexes"""
        self.assertRaises(ValueError,
                          self.manager.ifDescr.__setitem__, (47, 18), "Nooo")
        self.assertRaises(ValueError,
                          self.manager.ifDescr.__setitem__,
                          "nothing", "Neither")


class TestManagerWithNone(TestManagerGet):

    """Test a manager answering None for inexistent stuff"""

    def setUp(self):
        self.manager = Manager(host="127.0.0.1:{}".format(self.agent.port),
                               community="public",
                               version=2, none=True)
        self.session = self.manager._session._session

    def testGetInexistentStuff(self):
        """Try to access stuff that does not exist on the agent"""
        self.assertEqual(self.manager.snimpyNotImplemented, None)
        self.assertEqual(self.manager.ifName[47], None)
        self.assertEqual(self.manager.ifDescr[47], None)


class TestCachingManager(TestManagerGet):

    """Test if caching manager works like regular manager"""

    def setUp(self):
        self.manager = Manager(host="127.0.0.1:{}".format(self.agent.port),
                               community="public",
                               version=2, cache=1)
        self.session = self.manager._session._session

    def testGetChangingStuff(self):
        """Get stuff with varying values"""
        initial = self.manager.ifInOctets[2]
        current = self.manager.ifInOctets[2]
        self.assertEqual(current, initial)

    def testCacheFlush(self):
        """Test cache timeout is working as expected"""
        first1 = self.manager.ifInOctets[1]
        second1 = self.manager.ifInOctets[2]
        third1 = self.manager.ifInOctets[3]
        time.sleep(0.5)
        second2 = self.manager.ifInOctets[2]
        third2 = self.manager.ifInOctets[3]
        self.assertEqual(second1, second2)  # timeout not reached
        self.assertEqual(third1, third2)  # timeout not reached
        time.sleep(1)
        first2 = self.manager.ifInOctets[1]
        self.assertGreater(first2, first1)  # timeout was reached


class TestCachingManagerWithModificatons(TestManager):

    """Test if caching manager works with modifications"""

    def setUp(self):
        self.manager = Manager(host="127.0.0.1:{}".format(self.agent.port),
                               community="public",
                               version=2, cache=1)
        self.session = self.manager._session._session

    def testCacheScalar(self):
        """Check that a scalar value is kept in cache"""
        original = self.manager.snimpyString
        self.manager.snimpyString = "Nooooo"
        self.assertEqual(self.manager.snimpyString, original)

    def testCacheNonScalar(self):
        """Check we can cache a non-scalar value"""
        idx = ("row2", (1, 0, 2, 3), "beta32", "end of row2")
        original = self.manager.snimpyIndexInt[idx]
        self.manager.snimpyIndexInt[idx] = 1041
        self.assertEqual(self.manager.snimpyIndexInt[idx], original)

    def testCacheExpire(self):
        """Check the cache can expire"""
        self.manager.snimpyString = "Yeesss"
        time.sleep(1)
        self.assertEqual(self.manager.snimpyString, "Yeesss")


class TestManagerInvalidValues(TestManager):

    """Test when the agent is returning invalid values"""

    def testInvalidValue(self):
        """Check if an invalid value raises an exception"""
        self.assertRaises(ValueError,
                          getattr, self.manager,
                          "snimpyMacAddressInvalid")

    def testInvalidValueInTable(self):
        """Check if an invalid value in a table raises an exception"""
        self.assertRaises(ValueError,
                          self.manager.snimpyInvalidDescr.__getitem__,
                          2)

    def testInvalidValueWhileIterating(self):
        """Check if an invalid value while walking raises an exception"""
        self.assertRaises(ValueError,
                          list,
                          self.manager.snimpyInvalidDescr.iteritems())


class TestManagerLoose(TestManager):

    """Test when the agent is returning invalid values with loose mode"""

    def setUp(self):
        self.manager = Manager(host="127.0.0.1:{}".format(self.agent.port),
                               community="public",
                               version=2, loose=True)
        self.session = self.manager._session

    def testInvalidValue(self):
        """Check if an invalid value is correctly returned"""
        self.assertEqual(self.manager.snimpyMacAddressInvalid,
                         b"\xf1\x12\x13\x14\x15\x16")

    def testInvalidValueInTable(self):
        """Check if an invalid value in a table is correctly returned"""
        self.assertEqual(self.manager.snimpyInvalidDescr[1],
                         "Hello")
        self.assertEqual(self.manager.snimpyInvalidDescr[2],
                         b"\xf1\x12\x13\x14\x15\x16")

    def testInvalidValueWhileIterating(self):
        """Check if an invalid value while walking works"""
        self.assertEqual(list(self.manager.snimpyInvalidDescr.iteritems()),
                         [(1, "Hello"),
                          (2, b"\xf1\x12\x13\x14\x15\x16")])



#!/usr/bin/snimpy

"""
Set NTP or syslog server on various switches

Usage:
 ./set-syslog-ntp.py [syslog|ntp] host community first [...]
"""

import os
import sys

load("SNMPv2-MIB")

host = sys.argv[2]
targets = sys.argv[4:]
operation = sys.argv[1]

try:
    s = M(host=host, community=sys.argv[3])
    sid = str(s.sysObjectID)
except snmp.SNMPException as e:
    print("%s: %s" % (host, e))
    sys.exit(1)

if sid.startswith("1.3.6.1.4.1.45.3."):
    # Nortel
    print("%s is Nortel 55xx" % host)
    load(os.path.expanduser("~/.snmp/mibs/SYNOPTICS-ROOT-MIB"))
    load(os.path.expanduser("~/.snmp/mibs/S5-ROOT-MIB"))
    if operation == "ntp":
        load(os.path.expanduser("~/.snmp/mibs/S5-AGENT-MIB"))
        s.s5AgSntpPrimaryServerAddress = targets[0]
        if len(targets) > 1:
            s.s5AgSntpSecondaryServerAddress = targets[1]
        else:
            s.s5AgSntpSecondaryServerAddress = "0.0.0.0"
        s.s5AgSntpState = "unicast"
        s.s5AgSntpManualSyncRequest = "requestSync"
    elif operation == "syslog":
        load(os.path.expanduser("~/.snmp/mibs/BN-LOG-MESSAGE-MIB"))
        s.bnLogMsgRemoteSyslogAddress = targets[0]
        s.bnLogMsgRemoteSyslogSaveTargets = "msgTypeInformational"
        s.bnLogMsgRemoteSyslogEnabled = True
elif sid.startswith("1.3.6.1.4.1.1872."):
    print("%s is Alteon" % host)
    load(os.path.expanduser("~/.snmp/mibs/ALTEON-ROOT-MIB"))
    if operation == "ntp":
        s.agNewCfgNTPServer = targets[0]
        if len(targets) > 1:
            s.agNewCfgNTPSecServer = targets[1]
        else:
            s.agNewCfgNTPSecServer = "0.0.0.0"
        s.agNewCfgNTPService = "enabled"
    elif operation == "syslog":
        s.agNewCfgSyslogHost = targets[0]
        s.agNewCfgSyslogFac = "local2"
        if len(targets) > 1:
            s.agNewCfgSyslog2Host = targets[1]
            s.agNewCfgSyslog2Fac = "local2"
        else:
            s.agNewCfgSyslog2Host = "0.0.0.0"
    if s.agApplyPending == "applyNeeded":
        if s.agApplyConfig == "complete":
            s.agApplyConfig = "idle"
        s.agApplyConfig = "apply"
else:
    print("%s is unknown (%s)" % (host, s.sysDescr))
    sys.exit(1)



#!/usr/bin/snimpy

"""
On Nortel switches, list vlan on all active ports
"""

import os
import sys

load("SNMPv2-MIB")
load("IF-MIB")
load(os.path.expanduser("~/.snmp/mibs/RAPID-CITY-MIB"))
load(os.path.expanduser("~/.snmp/mibs/RC-VLAN-MIB"))

s = M(host=sys.argv[1], community=sys.argv[2])

vlans = {}

for interface in s.ifIndex:
    if s.ifOperStatus[interface] == "up":
        vlans[int(interface)] = []

for vlan in s.rcVlanId:
    for interface in vlans:
        if s.rcVlanStaticMembers[vlan] & interface:
            vlans[interface].append("{}({})".format(vlan, s.rcVlanName[vlan]))

import pprint
pprint.pprint(vlans)




import os
import tempfile
import code  # noqa: F401
import platform
from snimpy.main import interact
from multiprocessing import Process
import agent
import unittest
import unittest.mock as mock


class TestMain(unittest.TestCase):

    """Test the main shell"""

    @classmethod
    def setUpClass(cls):
        cls.agent = agent.TestAgent()

    @classmethod
    def tearDownClass(cls):
        cls.agent.terminate()

    @unittest.skipIf(platform.python_implementation() == "PyPy",
                     "setupterm seems unreliable with Pypy")
    def test_loadfile(self):
        script = tempfile.NamedTemporaryFile(delete=False)
        try:
            script.write("""
load("IF-MIB")
m = M(host="127.0.0.1:{}",
      community="public",
      version=2)
assert(m.ifDescr[1] == "lo")
""".format(self.agent.port).encode("ascii"))
            script.close()
            with mock.patch("code.InteractiveInterpreter.write"):
                p = Process(target=interact, args=((script.name,),))
                p.start()
                p.join()
                self.assertEqual(p.exitcode, 0)
        finally:
            os.unlink(script.name)



from multiprocessing import Process, Queue
import random

from pysnmp.entity import engine, config
from pysnmp.entity.rfc3413 import cmdrsp, context
from pysnmp.carrier.asyncio.dgram import udp, udp6
from pysnmp.proto.api import v2c


class TestAgent:

    next_port = [random.randint(22000, 32000)]

    """Agent for testing purpose"""

    def __init__(self, ipv6=False, community='public',
                 authpass='authpass', privpass='privpass',
                 emptyTable=True):
        q = Queue()
        self.ipv6 = ipv6
        self.emptyTable = emptyTable
        self.community = community
        self.authpass = authpass
        self.privpass = privpass
        self.next_port[0] += 1
        self._process = Process(target=self._setup,
                                args=(q, self.next_port[0]))
        self._process.start()
        self.port = q.get()

    def terminate(self):
        self._process.terminate()

    def _setup(self, q, port):
        """Setup a new agent in a separate process.

        The port the agent is listening too will be returned using the
        provided queue.
        """
        snmpEngine = engine.SnmpEngine()
        if self.ipv6:
            config.addSocketTransport(
                snmpEngine,
                udp6.domainName,
                udp6.Udp6Transport().openServerMode(('::1', port)))
        else:
            config.addSocketTransport(
                snmpEngine,
                udp.domainName,
                udp.UdpTransport().openServerMode(('127.0.0.1', port)))
        # Community is public and MIB is writable
        config.addV1System(snmpEngine, 'read-write', self.community)
        config.addVacmUser(snmpEngine, 1, 'read-write', 'noAuthNoPriv',
                           (1, 3, 6), (1, 3, 6))
        config.addVacmUser(snmpEngine, 2, 'read-write', 'noAuthNoPriv',
                           (1, 3, 6), (1, 3, 6))
        config.addV3User(
            snmpEngine, 'read-write',
            config.usmHMACMD5AuthProtocol, self.authpass,
            config.usmAesCfb128Protocol, self.privpass)
        config.addVacmUser(snmpEngine, 3, 'read-write', 'authPriv',
                           (1, 3, 6), (1, 3, 6))

        # Build MIB
        def stringToOid(string):
            return [ord(x) for x in string]

        def flatten(*args):
            result = []
            for el in args:
                if isinstance(el, (list, tuple)):
                    for sub in el:
                        result.append(sub)
                else:
                    result.append(el)
            return tuple(result)
        snmpContext = context.SnmpContext(snmpEngine)
        mibBuilder = snmpContext.getMibInstrum().getMibBuilder()
        (MibTable, MibTableRow, MibTableColumn,
         MibScalar, MibScalarInstance) = mibBuilder.importSymbols(
            'SNMPv2-SMI',
            'MibTable', 'MibTableRow', 'MibTableColumn',
            'MibScalar', 'MibScalarInstance')

        class RandomMibScalarInstance(MibScalarInstance):
            previous_value = 0

            def getValue(self, name, idx):
                self.previous_value += random.randint(1, 2000)
                return self.getSyntax().clone(self.previous_value)

        mibBuilder.exportSymbols(
            '__MY_SNMPv2_MIB',
            # SNMPv2-MIB::sysDescr
            MibScalar((1, 3, 6, 1, 2, 1, 1, 1), v2c.OctetString()),
            MibScalarInstance((1, 3, 6, 1, 2, 1, 1, 1), (0,),
                              v2c.OctetString(
                                  "Snimpy Test Agent {}".format(
                                      self.community))),
            # SNMPv2-MIB::sysObjectID
            MibScalar((1, 3, 6, 1, 2, 1, 1, 2), v2c.ObjectIdentifier()),
            MibScalarInstance((1, 3, 6, 1, 2, 1, 1, 2), (0,),
                              v2c.ObjectIdentifier((1, 3, 6, 1, 4,
                                                    1, 9, 1, 1208))))
        mibBuilder.exportSymbols(
            '__MY_IF_MIB',
            # IF-MIB::ifNumber
            MibScalar((1, 3, 6, 1, 2, 1, 2, 1), v2c.Integer()),
            MibScalarInstance((1, 3, 6, 1, 2, 1, 2, 1), (0,), v2c.Integer(3)),
            # IF-MIB::ifTable
            MibTable((1, 3, 6, 1, 2, 1, 2, 2)),
            MibTableRow((1, 3, 6, 1, 2, 1, 2, 2, 1)).setIndexNames(
                (0, '__MY_IF_MIB', 'ifIndex')),
            # IF-MIB::ifIndex
            MibScalarInstance(
                (1, 3, 6, 1, 2, 1, 2, 2, 1, 1), (1,), v2c.Integer(1)),
            MibScalarInstance(
                (1, 3, 6, 1, 2, 1, 2, 2, 1, 1), (2,), v2c.Integer(2)),
            MibScalarInstance(
                (1, 3, 6, 1, 2, 1, 2, 2, 1, 1), (3,), v2c.Integer(3)),
            # IF-MIB::ifDescr
            MibTableColumn((1, 3, 6, 1, 2, 1, 2, 2, 1, 2), v2c.OctetString()),
            MibScalarInstance(
                (1, 3, 6, 1, 2, 1, 2, 2, 1, 2), (1,), v2c.OctetString("lo")),
            MibScalarInstance(
                (1, 3, 6, 1, 2, 1, 2, 2, 1, 2), (2,), v2c.OctetString("eth0")),
            MibScalarInstance(
                (1, 3, 6, 1, 2, 1, 2, 2, 1, 2), (3,), v2c.OctetString("eth1")),
            # IF-MIB::ifType
            MibTableColumn((1, 3, 6, 1, 2, 1, 2, 2, 1, 3), v2c.Integer()),
            MibScalarInstance(
                (1, 3, 6, 1, 2, 1, 2, 2, 1, 3), (1,), v2c.Integer(24)),
            MibScalarInstance(
                (1, 3, 6, 1, 2, 1, 2, 2, 1, 3), (2,), v2c.Integer(6)),
            MibScalarInstance(
                (1, 3, 6, 1, 2, 1, 2, 2, 1, 3), (3,), v2c.Integer(6)),
            # IF-MIB::ifInOctets
            MibTableColumn((1, 3, 6, 1, 2, 1, 2, 2, 1, 10), v2c.Integer()),
            RandomMibScalarInstance(
                (1, 3, 6, 1, 2, 1, 2, 2, 1, 10), (1,), v2c.Gauge32()),
            RandomMibScalarInstance(
                (1, 3, 6, 1, 2, 1, 2, 2, 1, 10), (2,), v2c.Gauge32()),
            RandomMibScalarInstance(
                (1, 3, 6, 1, 2, 1, 2, 2, 1, 10), (3,), v2c.Gauge32()),

            # IF-MIB::ifRcvAddressTable
            MibTable((1, 3, 6, 1, 2, 1, 31, 1, 4)),
            MibTableRow((1, 3, 6, 1, 2, 1, 31, 1, 4, 1)).setIndexNames(
                (0, '__MY_IF_MIB', 'ifIndex'),
                (1, '__MY_IF_MIB', 'ifRcvAddressAddress')),
            # IF-MIB::ifRcvAddressStatus
            MibTableColumn((1, 3, 6, 1, 2, 1, 31, 1, 4, 1, 2), v2c.Integer()),
            MibScalarInstance(
                (1, 3, 6, 1, 2, 1, 31, 1, 4, 1, 2),
                flatten(2, 6, stringToOid("abcdef")), v2c.Integer(1)),
            MibScalarInstance(
                (1, 3, 6, 1, 2, 1, 31, 1, 4, 1, 2),
                flatten(2, 6, stringToOid("ghijkl")), v2c.Integer(1)),
            MibScalarInstance(
                (1, 3, 6, 1, 2, 1, 31, 1, 4, 1, 2),
                flatten(3, 6, stringToOid("mnopqr")), v2c.Integer(1)),
            # IF-MIB::ifRcvAddressType
            MibTableColumn((1, 3, 6, 1, 2, 1, 31, 1, 4, 1, 3), v2c.Integer()),
            MibScalarInstance(
                (1, 3, 6, 1, 2, 1, 31, 1, 4, 1, 3),
                flatten(2, 6, stringToOid("abcdef")), v2c.Integer(1)),
            MibScalarInstance(
                (1, 3, 6, 1, 2, 1, 31, 1, 4, 1, 3),
                flatten(2, 6, stringToOid("ghijkl")), v2c.Integer(1)),
            MibScalarInstance(
                (1, 3, 6, 1, 2, 1, 31, 1, 4, 1, 3),
                flatten(3, 6, stringToOid("mnopqr")), v2c.Integer(1)),

            # IF-MIB::ifIndex
            ifIndex=MibTableColumn((1, 3, 6, 1, 2, 1, 2, 2, 1, 1),
                                   v2c.Integer()),
            # IF-MIB::ifRcvAddressAddress
            ifRcvAddressAddress=MibTableColumn((1, 3, 6, 1, 2, 1, 31,
                                                1, 4, 1, 1),
                                               v2c.OctetString()))

        args = (
            '__MY_SNIMPY-MIB',
            # SNIMPY-MIB::snimpyIpAddress
            MibScalar((1, 3, 6, 1, 2, 1, 45121, 1, 1),
                      v2c.OctetString()).setMaxAccess("readwrite"),
            MibScalarInstance(
                (1, 3, 6, 1, 2, 1, 45121, 1, 1), (0,),
                v2c.OctetString("AAAA")),
            # SNIMPY-MIB::snimpyString
            MibScalar((1, 3, 6, 1, 2, 1, 45121, 1, 2),
                      v2c.OctetString()).setMaxAccess("readwrite"),
            MibScalarInstance(
                (1, 3, 6, 1, 2, 1, 45121, 1, 2), (0,), v2c.OctetString("bye")),
            # SNIMPY-MIB::snimpyInteger
            MibScalar((1, 3, 6, 1, 2, 1, 45121, 1, 3),
                      v2c.Integer()).setMaxAccess("readwrite"),
            MibScalarInstance(
                (1, 3, 6, 1, 2, 1, 45121, 1, 3), (0,), v2c.Integer(19)),
            # SNIMPY-MIB::snimpyEnum
            MibScalar((1, 3, 6, 1, 2, 1, 45121, 1, 4),
                      v2c.Integer()).setMaxAccess("readwrite"),
            MibScalarInstance(
                (1, 3, 6, 1, 2, 1, 45121, 1, 4), (0,), v2c.Integer(2)),
            # SNIMPY-MIB::snimpyObjectId
            MibScalar((1, 3, 6, 1, 2, 1, 45121, 1, 5),
                      v2c.ObjectIdentifier()).setMaxAccess("readwrite"),
            MibScalarInstance((1, 3, 6, 1, 2, 1, 45121, 1, 5), (
                0,), v2c.ObjectIdentifier((1, 3, 6, 4454, 0, 0))),
            # SNIMPY-MIB::snimpyBoolean
            MibScalar((1, 3, 6, 1, 2, 1, 45121, 1, 6),
                      v2c.Integer()).setMaxAccess("readwrite"),
            MibScalarInstance(
                (1, 3, 6, 1, 2, 1, 45121, 1, 6), (0,), v2c.Integer(1)),
            # SNIMPY-MIB::snimpyCounter
            MibScalar((1, 3, 6, 1, 2, 1, 45121, 1, 7),
                      v2c.Counter32()).setMaxAccess("readwrite"),
            MibScalarInstance(
                (1, 3, 6, 1, 2, 1, 45121, 1, 7), (0,), v2c.Counter32(47)),
            # SNIMPY-MIB::snimpyGauge
            MibScalar((1, 3, 6, 1, 2, 1, 45121, 1, 8),
                      v2c.Gauge32()).setMaxAccess("readwrite"),
            MibScalarInstance(
                (1, 3, 6, 1, 2, 1, 45121, 1, 8), (0,), v2c.Gauge32(18)),
            # SNIMPY-MIB::snimpyTimeticks
            MibScalar((1, 3, 6, 1, 2, 1, 45121, 1, 9),
                      v2c.TimeTicks()).setMaxAccess("readwrite"),
            MibScalarInstance(
                (1, 3, 6, 1, 2, 1, 45121, 1, 9), (0,),
                v2c.TimeTicks(12111100)),
            # SNIMPY-MIB::snimpyCounter64
            MibScalar((1, 3, 6, 1, 2, 1, 45121, 1, 10),
                      v2c.Counter64()).setMaxAccess("readwrite"),
            MibScalarInstance(
                (1, 3, 6, 1, 2, 1, 45121, 1, 10), (0,),
                v2c.Counter64(2 ** 48 + 3)),
            # SNIMPY-MIB::snimpyBits
            MibScalar((1, 3, 6, 1, 2, 1, 45121, 1, 11),
                      v2c.OctetString()).setMaxAccess("readwrite"),
            MibScalarInstance(
                (1, 3, 6, 1, 2, 1, 45121, 1, 11), (0,),
                v2c.OctetString(b"\xa0\x80")),
            # SNIMPY-MIB::snimpyMacAddress
            MibScalar((1, 3, 6, 1, 2, 1, 45121, 1, 15),
                      v2c.OctetString()).setMaxAccess("readwrite"),
            MibScalarInstance((1, 3, 6, 1, 2, 1, 45121, 1, 15), (
                0,), v2c.OctetString(b"\x11\x12\x13\x14\x15\x16")),
            # SNIMPY-MIB::snimpyMacAddressInvalid
            MibScalar((1, 3, 6, 1, 2, 1, 45121, 1, 16),
                      v2c.OctetString()).setMaxAccess("readwrite"),
            MibScalarInstance((1, 3, 6, 1, 2, 1, 45121, 1, 16), (
                0,), v2c.OctetString(b"\xf1\x12\x13\x14\x15\x16")),

            # SNIMPY-MIB::snimpyIndexTable
            MibTable((1, 3, 6, 1, 2, 1, 45121, 2, 3)),
            MibTableRow(
                (1, 3, 6, 1, 2, 1, 45121, 2, 3, 1)).setIndexNames(
                (0, "__MY_SNIMPY-MIB", "snimpyIndexVarLen"),
                (0, "__MY_SNIMPY-MIB", "snimpyIndexOidVarLen"),
                (0, "__MY_SNIMPY-MIB", "snimpyIndexFixedLen"),
                (1, "__MY_SNIMPY-MIB", "snimpyIndexImplied")),
            # SNIMPY-MIB::snimpyIndexVarLen
            MibScalarInstance((1, 3, 6, 1, 2, 1, 45121, 2, 3, 1, 1),
                              flatten(4, stringToOid('row1'),
                                      3, 1, 2, 3,
                                      stringToOid('alpha5'),
                                      stringToOid('end of row1')),
                              v2c.OctetString(b"row1")),
            MibScalarInstance((1, 3, 6, 1, 2, 1, 45121, 2, 3, 1, 1),
                              flatten(4, stringToOid('row2'),
                                      4, 1, 0, 2, 3,
                                      stringToOid('beta32'),
                                      stringToOid('end of row2')),
                              v2c.OctetString(b"row2")),
            MibScalarInstance((1, 3, 6, 1, 2, 1, 45121, 2, 3, 1, 1),
                              flatten(4, stringToOid('row3'),
                                      4, 120, 1, 2, 3,
                                      stringToOid('gamma7'),
                                      stringToOid('end of row3')),
                              v2c.OctetString(b"row3")),
            # SNIMPY-MIB::snimpyIndexInt
            MibScalarInstance((1, 3, 6, 1, 2, 1, 45121, 2, 3, 1, 6),
                              flatten(4, stringToOid('row1'),
                                      3, 1, 2, 3,
                                      stringToOid('alpha5'),
                                      stringToOid('end of row1')),
                              v2c.Integer(4571)),
            MibScalarInstance((1, 3, 6, 1, 2, 1, 45121, 2, 3, 1, 6),
                              flatten(4, stringToOid('row2'),
                                      4, 1, 0, 2, 3,
                                      stringToOid('beta32'),
                                      stringToOid('end of row2')),
                              v2c.Integer(78741)),
            MibScalarInstance((1, 3, 6, 1, 2, 1, 45121, 2, 3, 1, 6),
                              flatten(4, stringToOid('row3'),
                                      4, 120, 1, 2, 3,
                                      stringToOid('gamma7'),
                                      stringToOid('end of row3')),
                              v2c.Integer(4110)),

            # SNIMPY-MIB::snimpyReuseIndexTable
            MibTable((1, 3, 6, 1, 2, 1, 45121, 2, 7)),
            MibTableRow(
                (1, 3, 6, 1, 2, 1, 45121, 2, 7, 1)).setIndexNames(
                (0, "__MY_SNIMPY-MIB", "snimpyIndexImplied"),
                (0, "__MY_SNIMPY-MIB", "snimpySimpleIndex")),
            # SNIMPY-MIB::snimpyReuseIndexValue
            MibScalarInstance((1, 3, 6, 1, 2, 1, 45121, 2, 7, 1, 1),
                              flatten(11, stringToOid('end of row1'),
                                      4),
                              v2c.Integer(1785)),
            MibScalarInstance((1, 3, 6, 1, 2, 1, 45121, 2, 7, 1, 1),
                              flatten(11, stringToOid('end of row1'),
                                      5),
                              v2c.Integer(2458)),

            # SNIMPY-MIB::snimpyInvalidTable
            MibTable((1, 3, 6, 1, 2, 1, 45121, 2, 5)),
            MibTableRow(
                (1, 3, 6, 1, 2, 1, 45121, 2, 5, 1)).setIndexNames(
                (0, "__MY_SNIMPY-MIB", "snimpyInvalidIndex")),
            # SNIMPY-MIB::snimpyInvalidDescr
            MibScalarInstance((1, 3, 6, 1, 2, 1, 45121, 2, 5, 1, 2),
                              (1,),
                              v2c.OctetString(b"Hello")),
            MibScalarInstance((1, 3, 6, 1, 2, 1, 45121, 2, 5, 1, 2),
                              (2,),
                              v2c.OctetString(b"\xf1\x12\x13\x14\x15\x16")))

        if self.emptyTable:
            args += (
                # SNIMPY-MIB::snimpyEmptyTable
                MibTable((1, 3, 6, 1, 2, 1, 45121, 2, 6)),
                MibTableRow(
                    (1, 3, 6, 1, 2, 1, 45121, 2, 6, 1)).setIndexNames(
                        (0, "__MY_SNIMPY-MIB", "snimpyEmptyIndex")))

        kwargs = dict(
            # Indexes
            snimpyIndexVarLen=MibTableColumn(
                (1, 3, 6, 1, 2, 1, 45121, 2, 3, 1, 1),
                v2c.OctetString(
                )),
            snimpyIndexIntIndex=MibTableColumn(
                (1, 3, 6, 1, 2, 1, 45121, 2, 3, 1, 2),
                v2c.Integer(
                )).setMaxAccess(
                "noaccess"),
            snimpyIndexOidVarLen=MibTableColumn(
                (1, 3, 6, 1, 2, 1, 45121, 2, 3, 1, 3),
                v2c.ObjectIdentifier(
                )).setMaxAccess(
                "noaccess"),
            snimpyIndexFixedLen=MibTableColumn(
                (1, 3, 6, 1, 2, 1, 45121, 2, 3, 1, 4),
                v2c.OctetString(
                ).setFixedLength(
                    6)).setMaxAccess(
                "noaccess"),
            snimpyIndexImplied=MibTableColumn(
                (1, 3, 6, 1, 2, 1, 45121, 2, 3, 1, 5),
                v2c.OctetString(
                )).setMaxAccess("noaccess"),
            snimpyIndexInt=MibTableColumn(
                (1, 3, 6, 1, 2, 1, 45121, 2, 3, 1, 6),
                v2c.Integer()).setMaxAccess("readwrite"),
            snimpyInvalidIndex=MibTableColumn(
                (1, 3, 6, 1, 2, 1, 45121, 2, 5, 1, 1),
                v2c.Integer()).setMaxAccess("noaccess"),
            snimpyInvalidDescr=MibTableColumn(
                (1, 3, 6, 1, 2, 1, 45121, 2, 5, 1, 2),
                v2c.OctetString()).setMaxAccess("readwrite"),
            snimpyReuseIndexValue=MibTableColumn(
                (1, 3, 6, 1, 2, 1, 45121, 2, 7, 1, 1),
                v2c.Integer()).setMaxAccess("readwrite")
        )

        if self.emptyTable:
            kwargs.update(dict(
                snimpyEmptyIndex=MibTableColumn(
                    (1, 3, 6, 1, 2, 1, 45121, 2, 6, 1, 1),
                    v2c.Integer()).setMaxAccess("noaccess"),
                snimpyEmptyDescr=MibTableColumn(
                    (1, 3, 6, 1, 2, 1, 45121, 2, 6, 1, 2),
                    v2c.OctetString()).setMaxAccess("readwrite")))

        mibBuilder.exportSymbols(*args, **kwargs)

        # Start agent
        cmdrsp.GetCommandResponder(snmpEngine, snmpContext)
        cmdrsp.SetCommandResponder(snmpEngine, snmpContext)
        cmdrsp.NextCommandResponder(snmpEngine, snmpContext)
        cmdrsp.BulkCommandResponder(snmpEngine, snmpContext)
        q.put(port)
        snmpEngine.transportDispatcher.jobStarted(1)
        snmpEngine.transportDispatcher.runDispatcher()



import os
from setuptools import setup
from setuptools.command.test import test
import snimpy

rtd = os.environ.get("READTHEDOCS", None) == "True"


class SnimpyTestCommand(test):
    def run_tests(self, *args, **kwds):
        # Ensure we keep a reference to multiprocessing and pysnmp to
        # avoid errors at the end of the test
        import multiprocessing
        import pysnmp

        SnimpyTestCommand.multiprocessing = multiprocessing
        SnimpyTestCommand.pysnmp = pysnmp
        return test.run_tests(self, *args, **kwds)


if __name__ == "__main__":
    readme = open("README.rst").read()
    history = open("HISTORY.rst").read().replace(".. :changelog:", "")

    setup(
        name="snimpy",
        classifiers=[
            "Development Status :: 4 - Beta",
            "Environment :: Console",
            "Intended Audience :: System Administrators",
            "License :: OSI Approved :: ISC License (ISCL)",
            "Operating System :: POSIX",
            "Programming Language :: Python :: 3",
            "Topic :: System :: Networking",
            "Topic :: Utilities",
            "Topic :: System :: Monitoring",
        ],
        url="https://github.com/vincentbernat/snimpy",
        description=snimpy.__doc__,
        long_description=readme + "\n\n" + history,
        long_description_content_type="text/x-rst",
        author=snimpy.__author__,
        author_email=snimpy.__email__,
        packages=["snimpy"],
        entry_points={
            "console_scripts": [
                "snimpy = snimpy.main:interact",
            ],
        },
        data_files=[("share/man/man1", ["man/snimpy.1"])],
        zip_safe=False,
        cffi_modules=(not rtd and ["snimpy/smi_build.py:ffi"] or []),
        install_requires=[
            "cffi >= 1.0.0",
            "pysnmp-lextudio >= 4, < 6",
            'pyasyncore; python_version >= "3.12"',
            "setuptools",
        ],
        setup_requires=["cffi >= 1.0.0", "vcversioner"],
        cmdclass={"test": SnimpyTestCommand},
        pbr=False,
        vcversioner={
            "version_module_paths": ["snimpy/_version.py"],
        },
    )



#
# snimpy -- Interactive SNMP tool
#
# Copyright (C) Vincent Bernat <bernat@luffy.cx>
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

import os.path
import importlib


class Conf:
    prompt = "\033[1m[snimpy]>\033[0m "
    histfile = "~/.snimpy_history"  # Not used with IPython
    userconf = "~/.snimpy.conf"
    ipython = True
    ipythonprofile = None  # Set for example to "snimpy"
    mibs = []

    def load(self, userconf=None):
        if userconf is None:
            userconf = self.userconf
        loader = importlib.machinery.SourceFileLoader(
            'confuser',
            os.path.expanduser(userconf))
        spec = importlib.util.spec_from_loader('confuser', loader)
        if spec is not None:
            try:
                confuser = importlib.util.module_from_spec(spec)
                spec.loader.exec_module(confuser)
            except FileNotFoundError:
                pass
            else:
                for k in confuser.__dict__:
                    if not k.startswith("__"):
                        setattr(self, k, confuser.__dict__[k])

        return self



#
# snimpy -- Interactive SNMP tool
#
# Copyright (C) Vincent Bernat <bernat@luffy.cx>
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""
This module is aimed at providing Pythonic representation of
various SNMP types. Each SMIv2 type is mapped to a corresponding class
which tries to mimic a basic type from Python. For example, display
strings are like Python string while SMIv2 integers are just like
Python integers. This module is some kind of a hack and its use
outside of *Snimpy* seems convoluted.
"""

import struct
import re
import ipaddress
from datetime import timedelta
from pysnmp.proto import rfc1902

from snimpy import mib


def ordering_with_cmp(cls):
    ops = {'__lt__': lambda self, other: self.__cmp__(other) < 0,
           '__gt__': lambda self, other: self.__cmp__(other) > 0,
           '__le__': lambda self, other: self.__cmp__(other) <= 0,
           '__ge__': lambda self, other: self.__cmp__(other) >= 0,
           '__eq__': lambda self, other: self.__cmp__(other) == 0,
           '__ne__': lambda self, other: self.__cmp__(other) != 0}
    for opname, opfunc in ops.items():
        opfunc.__name__ = opname
        opfunc.__doc__ = getattr(int, opname).__doc__
        setattr(cls, opname, opfunc)
    return cls


class Type:

    """Base class for all types."""

    def __new__(cls, entity, value, raw=True):
        """Create a new typed value.

        :param entity: A :class:`mib.Node` instance
        :param value: The value to set
        :param raw: Whetever the raw value is provided (as opposed to
            a user-supplied value). This parameter is important when
            the provided input is ambiguous, for example when it is an
            array of bytes.
        :type raw: bool
        :return: an instance of the new typed value
        """
        if entity.type != cls:
            raise ValueError("MIB node is {}. We are {}".format(entity.type,
                                                                cls))

        if cls == OctetString and entity.fmt is not None:
            # Promotion of OctetString to String if we have unicode stuff
            if isinstance(value, (String, str)) or not raw:
                cls = String

        if not isinstance(value, Type):
            value = cls._internal(entity, value)
        else:
            value = cls._internal(entity, value._value)
        if issubclass(cls, str):
            self = str.__new__(cls, value)
        elif issubclass(cls, bytes):
            self = bytes.__new__(cls, value)
        elif issubclass(cls, int):
            self = int.__new__(cls, value)
        else:
            self = object.__new__(cls)

        self._value = value
        self.entity = entity

        if cls == OctetString and entity.fmt is not None:
            # A display-hint propose to use only ascii and UTF-8
            # chars. We promote an OCTET-STRING to a DisplayString if
            # we have a format. This means we won't be able to access
            # individual bytes in this format, only the full displayed
            # version.
            value = String._internal(entity, self)
            self = str.__new__(String, value)
            self._value = value
            self.entity = entity

        if isinstance(self, String):
            # Ensure that strings follow their format, if it is applied.
            # This is safer and simpler than toOid, as it does not do
            # additional validation.
            self._toBytes()

        return self

    def __init__(self, *args, **kwargs):
        # Neutralize __init__ from other inherited classes
        pass

    @classmethod
    def _internal(cls, entity, value):
        """Get internal value for a given value."""
        raise NotImplementedError  # pragma: no cover

    def pack(self):
        """Prepare the instance to be sent on the wire."""
        raise NotImplementedError  # pragma: no cover

    def toOid(self, implied=False):
        """Convert to an OID.

        If this function is implemented, then class function
        :meth:`fromOid` should also be implemented as the "invert"
        function of this one.

        This function only works if the entity is used as an index!
        Otherwise, it should raises NotImplementedError.

        :return: An OID that can be used as index
        """
        raise NotImplementedError  # pragma: no cover

    @classmethod
    def fromOid(cls, entity, oid, implied=False):
        """Create instance from an OID.

        This is the sister function of :meth:`toOid`.

        :param oid: The OID to use to create an instance
        :param entity: The MIB entity we want to instantiate
        :return: A couple `(l, v)` with `l` the number of suboids
           needed to create the instance and `v` the instance created from
           the OID
        """
        raise NotImplementedError  # pragma: no cover

    @classmethod
    def _fixedLen(cls, entity):
        """Determine if the given entity is fixed-len

        This function is a helper that is used for String and
        Oid. When converting a variable-length type to an OID, we need
        to prefix it by its len or not depending of what the MIB say.

        Node that the type can be used in an index with IMPLIED keyword.
        In that case, even when this function returns False, the OID
        will not be prefixed by its length.

        :param entity: entity to check
        :return: `True` if it is fixed-len, `False` otherwise
        """
        if entity.ranges and not isinstance(entity.ranges, (tuple, list)):
            # Fixed length
            return True
        else:
            # Variable length
            return False

    def __str__(self):
        return str(self._value)

    def __repr__(self):
        return '<{}: {}>'.format(self.__class__.__name__,
                                 str(self))


@ordering_with_cmp
class IpAddress(Type, ipaddress.IPv4Address):

    """Class representing an IP address/"""

    @classmethod
    def _internal(cls, entity, value):
        if isinstance(value, (list, tuple)):
            value = ".".join([str(a) for a in value])
        try:
            value = ipaddress.IPv4Address(value)
        except ipaddress.AddressValueError:
            raise ValueError("{!r} is not a valid IP".format(value))
        return value

    def pack(self):
        return rfc1902.IpAddress(str(self._value))

    def toOid(self, implied=False):
        return tuple(self._value.packed)

    @classmethod
    def fromOid(cls, entity, oid, implied=False):
        if len(oid) < 4:
            raise ValueError(
                "{!r} is too short for an IP address".format(oid))
        return (4, cls(entity, oid[:4]))

    def __cmp__(self, other):
        if not isinstance(other, IpAddress):
            try:
                other = IpAddress(self.entity, other)
            except Exception:
                raise NotImplementedError  # pragma: no cover
        if self._value == other._value:
            return 0
        if self._value < other._value:
            return -1
        return 1

    def __getitem__(self, nb):
        return self._value.packed[nb]


class StringOrOctetString(Type):

    def toOid(self, implied=False):
        # To convert properly to OID, we need to know if it is a
        # fixed-len string, an implied string or a variable-len
        # string.
        b = self._toBytes()

        if implied or self._fixedLen(self.entity):
            return tuple(b)
        else:
            return (len(b),) + tuple(b)

    def _toBytes(self):
        raise NotImplementedError

    @classmethod
    def fromOid(cls, entity, oid, implied=False):
        oid = tuple(o & 0xff for o in oid)
        if implied:
            # Eat everything
            return (len(oid), cls(entity, bytes(oid)))
        if cls._fixedLen(entity):
            length = entity.ranges
            if len(oid) < length:
                raise ValueError(
                    "{} is too short for wanted fixed "
                    "string (need at least {:d})".format(oid, length))
            return (length,
                    cls(entity, bytes(oid[:length])))

        # This is var-len
        if not oid:
            raise ValueError("empty OID while waiting for var-len string")
        length = oid[0]
        if len(oid) < length + 1:
            raise ValueError(
                "{} is too short for variable-len "
                "string (need at least {:d})".format(oid, length))
        return (
            (length + 1,
             cls(entity, bytes(oid[1:(length + 1)]))))

    def pack(self):
        return rfc1902.OctetString(self._toBytes())


class OctetString(StringOrOctetString, bytes):

    """Class for a generic octet string. This class should be compared to
    :class:`String` which is used to represent a display string. This
    class is usually used to store raw bytes, like a bitmask of
    VLANs.
    """

    @classmethod
    def _internal(cls, entity, value):
        # Internally, we are using bytes
        if isinstance(value, bytes):
            return value
        if isinstance(value, str):
            return value.encode("ascii")
        return bytes(value)

    def _toBytes(self):
        return self._value

    def __ior__(self, value):
        nvalue = bytearray(self._value)
        if not isinstance(value, (tuple, list)):
            value = [value]
        for v in value:
            if not isinstance(v, int):
                raise NotImplementedError(
                    "on string, bit-operation are limited to integers")
            if len(nvalue) < (v >> 3) + 1:
                nvalue.extend([0] * ((v >> 3) + 1 - len(self._value)))
            nvalue[v >> 3] |= 1 << (7 - v % 8)
        return self.__class__(self.entity, bytes(nvalue))

    def __isub__(self, value):
        nvalue = bytearray(self._value)
        if not isinstance(value, (tuple, list)):
            value = [value]
        for v in value:
            if not isinstance(v, int):
                raise NotImplementedError(
                    "on string, bit-operation are limited to integers")
            if len(nvalue) < (v >> 3) + 1:
                continue
            nvalue[v >> 3] &= ~(1 << (7 - v % 8))
        return self.__class__(self.entity, bytes(nvalue))
        return self

    def __and__(self, value):
        nvalue = bytearray(self._value)
        if not isinstance(value, (tuple, list)):
            value = [value]
        for v in value:
            if not isinstance(v, int):
                raise NotImplementedError(
                    "on string, bit-operation are limited to integers")
            if len(nvalue) < (v >> 3) + 1:
                return False
            if not (nvalue[v >> 3] & (1 << (7 - v % 8))):
                return False
        return True


class String(StringOrOctetString, str):

    """Class for a display string. Such a string is an unicode string and
    it is therefore expected that only printable characters are
    used. This is usually the case if the corresponding MIB node comes
    with a format string.

    With such an instance, the user is expected to be able to provide
    a formatted. For example, a MAC address could be written
    `00:11:22:33:44:55`.
    """

    @classmethod
    def _parseOctetFormat(cls, fmt, j):
        # repeater
        if fmt[j] == "*":
            dorepeat = True
            j += 1
        else:
            dorepeat = False

        # length
        length = ""
        while fmt[j].isdigit():
            length += fmt[j]
            j += 1
        length = int(length)

        # format
        format = fmt[j]
        j += 1

        # seperator
        if j < len(fmt) and \
                fmt[j] != "*" and not fmt[j].isdigit():
            sep = fmt[j]
            j += 1
        else:
            sep = ""

        # terminator
        if j < len(fmt) and \
                fmt[j] != "*" and not fmt[j].isdigit():
            term = fmt[j]
            j += 1
        else:
            term = ""

        return (j, dorepeat, length, format, sep, term)

    @classmethod
    def _fromBytes(cls, value, fmt):
        i = 0               # Position in value
        j = 0               # Position in fmt
        result = ""
        term = None
        sep = None
        while i < len(value):
            if j < len(fmt):
                j, dorepeat, length, format, sep, term = cls._parseOctetFormat(
                    fmt, j)

            # building
            if dorepeat:
                repeat = value[i]
                i += 1
            else:
                repeat = 1
            for r in range(repeat):
                bb = value[i:i + length]
                i += length
                if format in ['o', 'x', 'd']:
                    if length > 4:
                        raise ValueError(
                            "don't know how to handle integers "
                            "more than 4 bytes long")
                    bb = b"\x00" * (4 - length) + bb
                    number = struct.unpack(b"!l", bb)[0]
                    if format == "o":
                        # In Python2, oct() is 01242, while it is 0o1242 in
                        # Python3
                        result += "".join(oct(number).partition("o")[0:3:2])
                    elif format == "x":
                        result += hex(number)[2:]
                    else:       # format == "d":
                        result += str(number)
                elif format == "a":
                    result += bb.decode("ascii")
                elif format == "t":
                    result += bb.decode("utf-8")
                else:
                    raise ValueError("{!r} cannot be represented with "
                                     "the given display string ({})".format(
                                         bb, fmt))
                result += sep
            if sep and term:
                result = result[:-1]
            result += term
        if term or sep:
            result = result[:-1]
        return result

    def _toBytes(self):
        # We need to reverse what was done by `_fromBytes`. This is
        # not an exact science. In most case, this is easy because a
        # separator is used but sometimes, this is not. We do some
        # black magic that will fail.
        i = 0
        j = 0
        fmt = self.entity.fmt
        bb = b""
        while i < len(self._value):
            if j < len(fmt):
                parsed = self._parseOctetFormat(fmt, j)
                j, dorepeat, length, format, sep, term = parsed
                if format == "o":
                    fmatch = "(?P<o>[0-7]{{1,{0}}})".format(
                        int(length * 2.66667) + 1)
                elif format == "x":
                    fmatch = "(?P<x>[0-9A-Fa-f]{{1,{0}}})".format(length * 2)
                elif format == "d":
                    fmatch = "(?P<d>[0-9]{{1,{0}}})".format(
                        int(length * 2.4083) + 1)
                elif format == "a":
                    fmatch = "(?P<a>(?:.|\n){{1,{0}}})".format(length)
                elif format == "t":
                    fmatch = "(?P<t>(?:.|\n){{1,{0}}})".format(length)
                else:
                    raise ValueError("{!r} cannot be parsed due to an "
                                     "incorrect format ({})".format(
                                         self._value, fmt))
            repeats = []
            while True:
                mo = re.match(fmatch, self._value[i:])
                if not mo:
                    raise ValueError("{!r} cannot be parsed because it "
                                     "does not match format {} at "
                                     "index {}".format(self._value, fmt, i))
                if format in ["o", "x", "d"]:
                    if format == "o":
                        r = int(mo.group("o"), 8)
                    elif format == "x":
                        r = int(mo.group("x"), 16)
                    else:
                        r = int(mo.group("d"))
                    result = struct.pack(b"!l", r)[-length:]
                else:
                    result = mo.group(1).encode("utf-8")
                i += len(mo.group(1))
                if dorepeat:
                    repeats.append(result)
                    if i < len(self._value):
                        # Approximate...
                        if sep and self._value[i] == sep:
                            i += 1
                        elif term and self._value[i] == term:
                            i += 1
                            break
                    else:
                        break
                else:
                    break
            if dorepeat:
                bb += bytes([len(repeats)])
                bb += b"".join(repeats)
            else:
                bb += result
                if i < len(self._value) and (sep and self._value[i] == sep or
                                             term and self._value[i] == term):
                    i += 1

        return bb

    @classmethod
    def _internal(cls, entity, value):
        # Internally, we use the displayed string. We have a special
        # case if the value is an OctetString to do the conversion.
        if isinstance(value, OctetString):
            return cls._fromBytes(value._value, entity.fmt)
        if isinstance(value, bytes):
            return value.decode("utf-8")
        return str(value)

    def __str__(self):
        return self._value


class Integer(Type, int):

    """Class for any integer."""

    @classmethod
    def _internal(cls, entity, value):
        return int(value)

    def pack(self):
        if self._value >= (1 << 64):
            raise OverflowError("too large to be packed")
        if self._value >= (1 << 32):
            return rfc1902.Counter64(self._value)
        if self._value >= 0:
            return rfc1902.Integer(self._value)
        if self._value >= -(1 << 31):
            return rfc1902.Integer(self._value)
        raise OverflowError("too small to be packed")

    def toOid(self, implied=False):
        return (self._value,)

    @classmethod
    def fromOid(cls, entity, oid, implied=False):
        if len(oid) < 1:
            raise ValueError("{} is too short for an integer".format(oid))
        return (1, cls(entity, oid[0]))

    def __str__(self):
        if self.entity.fmt:
            if self.entity.fmt[0] == "x":
                return hex(self._value)
            if self.entity.fmt[0] == "o":
                return oct(self._value)
            if self.entity.fmt[0] == "b":
                if self._value == 0:
                    return "0"
                if self._value > 0:
                    v = self._value
                    r = ""
                    while v > 0:
                        r = str(v % 2) + r
                        v = v >> 1
                    return r
            elif self.entity.fmt[0] == "d" and \
                    len(self.entity.fmt) > 2 and \
                    self.entity.fmt[1] == "-":
                dec = int(self.entity.fmt[2:])
                result = str(self._value)
                if len(result) < dec + 1:
                    result = "0" * (dec + 1 - len(result)) + result
                return "{}.{}".format(result[:-2], result[-2:])
        return str(self._value)


class Unsigned32(Integer):

    """Class to represent an unsigned 32bits integer."""

    def pack(self):
        if self._value >= (1 << 32):
            raise OverflowError("too large to be packed")
        if self._value < 0:
            raise OverflowError("too small to be packed")
        return rfc1902.Unsigned32(self._value)


class Unsigned64(Integer):

    """Class to represent an unsigned 64bits integer."""

    def pack(self):
        if self._value >= (1 << 64):
            raise OverflowError("too large to be packed")
        if self._value < 0:
            raise OverflowError("too small to be packed")
        return rfc1902.Counter64(self._value)


class Enum(Integer):

    """Class for an enumeration. An enumaration is an integer but labels
    are attached to some values for a more user-friendly display."""

    @classmethod
    def _internal(cls, entity, value):
        if value in entity.enum:
            return value
        for (k, v) in entity.enum.items():
            if (v == value):
                return k
        try:
            return int(value)
        except Exception:
            raise ValueError("{!r} is not a valid "
                             "value for {}".format(value,
                                                   entity))

    def pack(self):
        return rfc1902.Integer(self._value)

    @classmethod
    def fromOid(cls, entity, oid, implied=False):
        if len(oid) < 1:
            raise ValueError(
                "{!r} is too short for an enumeration".format(oid))
        return (1, cls(entity, oid[0]))

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            try:
                other = self.__class__(self.entity, other)
            except Exception:
                raise NotImplementedError  # pragma: no cover
        return self._value == other._value

    def __ne__(self, other):
        return not self.__eq__(other)

    def __str__(self):
        if self._value in self.entity.enum:
            return (
                "{}({:d})".format(self.entity.enum[self._value], self._value)
            )
        else:
            return str(self._value)


@ordering_with_cmp
class Oid(Type):

    """Class to represent and OID."""

    @classmethod
    def _internal(cls, entity, value):
        if isinstance(value, (list, tuple)):
            return tuple(int(v) for v in value)
        elif isinstance(value, str):
            return tuple(int(i) for i in value.split(".") if i)
        elif isinstance(value, mib.Node):
            return tuple(value.oid)
        else:
            raise TypeError(
                "don't know how to convert {!r} to OID".format(value))

    def pack(self):
        return rfc1902.univ.ObjectIdentifier(self._value)

    def toOid(self, implied=False):
        if implied or self._fixedLen(self.entity):
            return self._value
        else:
            return tuple([len(self._value)] + list(self._value))

    @classmethod
    def fromOid(cls, entity, oid, implied=False):
        if cls._fixedLen(entity):
            # A fixed OID? We don't like this. Provide a real example.
            raise ValueError(
                "{!r} seems to be a fixed-len OID index. Odd.".format(entity))
        if not implied:
            # This index is not implied. We need the len
            if len(oid) < 1:
                raise ValueError(
                    "{!r} is too short for a not "
                    "implied index".format(entity))
            length = oid[0]
            if len(oid) < length + 1:
                raise ValueError(
                    "{!r} has an incorrect size "
                    "(needs at least {:d})".format(oid, length))
            return (length + 1, cls(entity, oid[1:(length + 1)]))
        else:
            # This index is implied. Eat everything
            return (len(oid), cls(entity, oid))

    def __str__(self):
        return ".".join([str(x) for x in self._value])

    def __cmp__(self, other):
        if not isinstance(other, Oid):
            other = Oid(self.entity, other)
        if tuple(self._value) == tuple(other._value):
            return 0
        if self._value > other._value:
            return 1
        return -1

    def __getitem__(self, index):
        return self._value[index]

    def __contains__(self, item):
        """Test if item is a sub-oid of this OID"""
        if not isinstance(item, Oid):
            item = Oid(self.entity, item)
        return tuple(item._value[:len(self._value)]) == \
            tuple(self._value[:len(self._value)])


class Boolean(Enum):

    """Class for a boolean."""

    @classmethod
    def _internal(cls, entity, value):
        if isinstance(value, bool):
            if value:
                return Enum._internal(entity, "true")
            else:
                return Enum._internal(entity, "false")
        else:
            return Enum._internal(entity, value)

    def __nonzero__(self):
        if self._value == 1:
            return True
        else:
            return False

    def __bool__(self):
        return self.__nonzero__()


@ordering_with_cmp
class Timeticks(Type):

    """Class for timeticks."""

    @classmethod
    def _internal(cls, entity, value):
        if isinstance(value, int):
            # Value in centiseconds
            return timedelta(0, value / 100.)
        elif isinstance(value, timedelta):
            return value
        else:
            raise TypeError(
                "dunno how to handle {!r} ({})".format(value, type(value)))

    def __int__(self):
        return self._value.days * 3600 * 24 * 100 + \
            self._value.seconds * 100 + \
            self._value.microseconds // 10000

    def toOid(self, implied=False):
        return (int(self),)

    @classmethod
    def fromOid(cls, entity, oid, implied=False):
        if len(oid) < 1:
            raise ValueError("{!r} is too short for a timetick".format(oid))
        return (1, cls(entity, oid[0]))

    def pack(self):
        return rfc1902.TimeTicks(int(self))

    def __str__(self):
        return str(self._value)

    def __cmp__(self, other):
        if isinstance(other, Timeticks):
            other = other._value
        elif isinstance(other, int):
            other = timedelta(0, other / 100.)
        elif not isinstance(other, timedelta):
            raise NotImplementedError(
                "only compare to int or "
                "timedelta, not {}".format(type(other)))
        if self._value == other:
            return 0
        if self._value < other:
            return -1
        return 1


class Bits(Type):

    """Class for bits."""

    @classmethod
    def _internal(cls, entity, value):
        bits = set()
        tryalternate = False
        if isinstance(value, bytes):
            for i, x in enumerate(value):
                if x == 0:
                    continue
                for j in range(8):
                    if x & (1 << (7 - j)):
                        k = (i * 8) + j
                        if k not in entity.enum:
                            tryalternate = True
                            break
                        bits.add(k)
                if tryalternate:
                    break
            if not tryalternate:
                return bits
            else:
                bits = set()
        elif not isinstance(value, (tuple, list, set, frozenset)):
            value = {value}
        for v in value:
            found = False
            if v in entity.enum:
                bits.add(v)
                found = True
            else:
                for (k, t) in entity.enum.items():
                    if (t == v):
                        bits.add(k)
                        found = True
                        break
            if not found:
                raise ValueError("{!r} is not a valid bit value".format(v))
        return bits

    def pack(self):
        if self._value:
            string = [0] * ((max(self._value) // 8) + 1)
        else:
            string = []
        for b in self._value:
            string[b // 8] |= 1 << (7 - b % 8)
        return rfc1902.Bits(bytes(string))

    def __eq__(self, other):
        if isinstance(other, str):
            other = [other]
        if not isinstance(other, Bits):
            other = Bits(self.entity, other)
        return self._value == other._value

    def __ne__(self, other):
        return not self.__eq__(other)

    def __str__(self):
        result = []
        for b in sorted(self._value):
            result.append("{}({:d})".format(self.entity.enum[b], b))
        return ", ".join(result)

    def __and__(self, other):
        if isinstance(other, str):
            other = [other]
        if not isinstance(other, Bits):
            other = Bits(self.entity, other)
        return len(self._value & other._value) > 0

    def __ior__(self, other):
        if isinstance(other, str):
            other = [other]
        if not isinstance(other, Bits):
            other = Bits(self.entity, other)
        self._value |= other._value
        return self

    def __isub__(self, other):
        if isinstance(other, str):
            other = [other]
        if not isinstance(other, Bits):
            other = Bits(self.entity, other)
        self._value -= other._value
        return self


def build(mibname, node, value):
    """Build a new basic type with the given value.

    :param mibname: The MIB to use to locate the entity.
    :param node: The node that will be attached to this type.
    :param value: The initial value to set for the type.
    :return: A :class:`Type` instance
    """
    m = mib.get(mibname, node)
    t = m.type(m, value)
    return t



#!/usr/bin/snimpy

"""Enable LLDP.

Generic procedure but we restrict ourself to Nortel 55x0.
"""

import sys
import os

load("SNMPv2-MIB")
for l in ["LLDP", "LLDP-EXT-DOT3", "LLDP-EXT-DOT1"]:
    load(os.path.expanduser("~/.snmp/mibs/%s-MIB" % l))

s = M(host=sys.argv[1], community=sys.argv[2])

try:
    type = s.sysDescr
except snmp.SNMPException:
    print("Cannot process %s: bad community?" % sys.argv[1])
    sys.exit(1)
if not type.startswith(("Ethernet Routing Switch 55",
                        "Ethernet Switch 425")):
    print("Not a 55x0: %s" % type)
    sys.exit(1)

print("Processing %s..." % sys.argv[1])
try:
    for oid in s.lldpConfigManAddrPortsTxEnable:
        if oid[0] == "ipV4":
            s.lldpConfigManAddrPortsTxEnable[oid] = "\xff"*10
except snmp.SNMPNoSuchObject:
    print("No LLDP for this switch")
    sys.exit(2)
dot3 = True
for port in s.lldpPortConfigAdminStatus:
    s.lldpPortConfigAdminStatus[port] = "txAndRx"
    s.lldpPortConfigTLVsTxEnable[port] = ["portDesc",
                                          "sysName",
                                          "sysDesc",
                                          "sysCap" ]
    # Dot3
    try:
        if dot3:
            s.lldpXdot3PortConfigTLVsTxEnable[port] = ["macPhyConfigStatus",
                                                       "powerViaMDI",
                                                       "linkAggregation",
                                                       "maxFrameSize"]
    except snmp.SNMPException:
        print("No Dot3")
        dot3 = False
# Dot1
try:
    for port,vlan in s.lldpXdot1ConfigVlanNameTxEnable:
        s.lldpXdot1ConfigVlanNameTxEnable[port, vlan] = True
except snmp.SNMPException:
    print("No Dot1")
print("Success!")
